# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

from zope.interface import implements

from BTrees.OOBTree import OOBTree
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.atapi import *
from Products.BlueWebMail.config import *
from Products.BlueWebMail.interfaces import IWebMail

schema = Schema((

    TextField(
        name='realname',
        widget=StringWidget(
            label="Real Name",
            label_msgid='BlueWebMail_label_relaname',
            i18n_domain='BlueWebMail',
        )
    ),
    
    TextField(
        name='fromAdress',
        widget=StringWidget(
            label="From Address",
            label_msgid='BlueWebMail_label_fromAddress',
            i18n_domain='BlueWebMail',
        )
    ),

    TextField(
        name='userName',
        widget=StringWidget(
            label="Username",
            label_msgid='BlueWebMail_label_userName',
            i18n_domain='BlueWebMail',
        )
    ),

    TextField(
        name='password',
        widget=PasswordWidget(
            label="Password",
            label_msgid='BlueWebMail_label_password',
            i18n_domain='BlueWebMail',
        )
    ),
    
    TextField(
        name='optionalImapServer',
        widget=StringWidget(
            label="Optional IMAP Server",
            label_msgid='BlueWebMail_label_optionalimapserver',
            i18n_domain='BlueWebMail',
        )
    ),
    
    TextField(
        name='optionalImapPort',
        widget=IntegerWidget(
            label="Optional IMAP Port",
            label_msgid='BlueWebMail_label_optionalimapport',
            i18n_domain='BlueWebMail',
        )
    ),
    
    TextField(
        name='optionalSmtpServer',
        widget=StringWidget(
            label="Optional SMTP Server",
            label_msgid='BlueWebMail_label_optionalsmtpserver',
            i18n_domain='BlueWebMail',
        )
    ),
    
    TextField(
        name='optionalSmtpPort',
        widget=IntegerWidget(
            label="Optional SMTP Port",
            label_msgid='BlueWebMail_label_optionalsmtpport',
            i18n_domain='BlueWebMail',
        )
    ),
    
    BooleanField(
        name='smtpRequiresLogin',
        default=False,
        widget=BooleanWidget(
            label="SMTP Server requires login",
            label_msgid='BlueWebMail_label_smtpserverrequireslogin',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    BooleanField(
        name='usePropertiesForAuthentication',
        default=False,
        widget=BooleanWidget(
            label="Use Properties from PAS for authentication",
            label_msgid='BlueWebMail_label_usepropertiesforauthentication',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    BooleanField(
        name='showMessagePartInformation',
        default=False,
        widget=BooleanWidget(
            label="Show message part information",
            label_msgid='BlueWebMail_label_showmessagepartinformation',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    IntegerField(
        name='mailCacheTimeout',
        default=300,
        widget=IntegerWidget(
            label="Mailcache Timout (Seconds)",
            label_msgid='BlueWebMail_label_mailcachetimeout',
            i18n_domain='BlueWebMail',
        )
    ),
    
    IntegerField(
        name='maxMessagesPerPage',
        default=15,
        widget=IntegerWidget(
            label="Number of Messages per Page",
            label_msgid='BlueWebMail_label_maxmessagesperpage',
            i18n_domain='BlueWebMail',
        )
    ),
    
    TextField(
        name='attachmentContainer',
        default='',
        widget=StringWidget(
            label="Attachment Container (defaults to self)",
            label_msgid='BlueWebMail_label_attachmentContainer',
            i18n_domain='BlueWebMail',
        )
    ),
    
    TextField(
        name='attachmentContainerType',
        default='',
        widget=StringWidget(
            label="Attachment Container Content Type (leave blank until ...)",
            label_msgid='BlueWebMail_label_attachmentContainerType',
            i18n_domain='BlueWebMail',
        )
    ),
    
    IntegerField(
        name='attachmentContainerMaxSize',
        default=50,
        widget=IntegerWidget(
            label="Max number of attachments in attachment Container.",
            label_msgid='BlueWebMail_label_attachmentContainerMaxSize',
            i18n_domain='BlueWebMail',
        )
    ),
    
    TextField(
        name='createFormat',
        default='HTML',
        widget=SelectionWidget(
            label='Preferred create format of emails',
            label_msgid='BlueWebMail_label_createFormat',
            i18n_domain='BlueWebMail',
        ),
        vocabulary=['HTML', 'TEXT'],
    ),
    
),
)

WebMailBox_schema = BaseSchema.copy() + \
    schema.copy()

class WebMailBox(BaseContent, BaseFolder):
    """In case of z2, we use an archetype for the mailbox properties storage.
    """
    
    implements(IWebMail)
    
    security = ClassSecurityInfo()
    __implements__ = (getattr(BaseContent,'__implements__',()),) + (getattr(BaseFolder,'__implements__',()),)

    archetype_name = 'MailBox'

    meta_type = 'WebMailBox'
    portal_type = 'WebMailBox'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 1
    content_icon = 'mailbox_icon.png'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "MailBox"
    typeDescMsgId = 'description_edit_mailbox'

    actions =  (
       {'action': "string:${object_url}/webmail_view",
        'category': "object",
        'id': 'view',
        'name': 'view',
        'permissions': ("View",),
        'condition': 'python:1'
       },
    )

    _at_rename_after_creation = True

    schema = WebMailBox_schema
    
    def __init__(self, *args, **kwargs):
        super(WebMailBox, self).__init__(*args, **kwargs)
        self.mailCache = OOBTree()
        
    security.declarePublic('showPortlet')
    def showPortlet(self):
        """This method sucks here.
        """
        path = self.REQUEST.get('PATH_TRANSLATED')
        path = path[path.rfind('/') + 1:]
        if path == 'view' \
          or path == 'webmail_view' \
          or path == 'message_view' \
          or path == 'create_message' \
          or path == self.id:
            return True
        return False
    
    security.declareProtected('View', 'getRealname')
    def getRealname(self, **kw):
        """Fetch field real-/full-name, if not set fall back is the webmailtool.        
        """
        field = self.getField('realname')
        name = field.get(self, **kw)
        if name:
            return name
        wmt = getToolByName(self, 'portal_webmailtool')
        name = wmt.getCurrentUsersProperty('fullname', name)
        return name

    security.declareProtected('View', 'getUserName')
    def getUserName(self, **kw):
        """Fetch field username, if not set falls back is the webmailtool.        
        """
        field = self.getField('userName')
        login = field.get(self, **kw)
        if login:
            return login
        wmt = getToolByName(self, 'portal_webmailtool')
        login = wmt.getCurrentUsersProperty('imaplogin', login)
        return login
    
    security.declareProtected('View', 'getPassword')
    def getPassword(self, **kw):
        """Fetch field password, if not set falls back is the webmailtool.        
        """
        field = self.getField('password')
        pw = field.get(self, **kw)
        if pw:
            return pw
        wmt = getToolByName(self, 'portal_webmailtool')
        pw = wmt.getCurrentUsersProperty('imappassword', pw)
        return pw

    security.declareProtected('View', 'getFromAdress')
    def getFromAdress(self, **kw):
        """Fetch field password, if not set falls back is the webmailtool.        
        """
        field = self.getField('fromAdress')
        faddr = field.get(self, **kw)
        if faddr:
            return faddr
        wmt = getToolByName(self, 'portal_webmailtool')
        faddr = wmt.getCurrentUsersProperty('email', faddr)
        return faddr

registerType(WebMailBox, PROJECTNAME)
 
