# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>,
                Jens Klein <jens@bluedynamics.com>"""
__docformat__ = 'plaintext'


from os import path as os_path

from Globals import DTMLFile
from Globals import package_home

from AccessControl import ClassSecurityInfo

from Products.Archetypes.atapi import *

from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.ActionProviderBase import ActionProviderBase

from Products.BlueWebMail.config import *
from Products.BlueWebMail.serversettings import IMAPSERVER_TYPES
from Products.BlueWebMail.serversettings import IMAPSERVER_PRESETS

schema = Schema((

    StringField(
        name='smtpServer',
        widget=StringWidget(
            label='SMTP Server',
            label_msgid='BlueWebMail_label_smtpServer',
            i18n_domain='BlueWebMail',
        ),
        default='localhost'
    ),
    
    StringField(
        name='smtpPort',
        widget=IntegerWidget(
            label='SMTP Port',
            label_msgid='BlueWebMail_label_smtpPort',
            i18n_domain='BlueWebMail',
        ),
        size=5,
        default='25'
    ),

    StringField(
        name='imapServer',
        widget=StringWidget(
            label='IMAP Server',
            label_msgid='BlueWebMail_label_imapServer',
            i18n_domain='BlueWebMail',
        ),
        default='localhost'
    ),
    
    StringField(
        name='imapPort',
        widget=IntegerWidget(
            label='IMAP Port',
            label_msgid='BlueWebMail_label_imapPort',
            i18n_domain='BlueWebMail',
        ),
        size=5,
        default='143'
    ),
    
    StringField(
        name='imapServerType',
        vocabulary='getImapServerTypesVocab',
        default='default',
        widget=SelectionWidget(
            label='IMAP Server Type',
            label_msgid='BlueWebMail_label_imapServerType',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    StringField(
        name='imapFolderPrefix',
        default='',
        widget=StringWidget(
            label='Folder prefix',
            label_msgid='BlueWebMail_label_imapFolderPrefix',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    StringField(
        name='imapTrashFolder',
        default='INBOX.Trash',
        widget=StringWidget(
            label='Trash folder',
            label_msgid='BlueWebMail_label_imapTrashFolder',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    StringField(
        name='imapSentFolder',
        default='INBOX.Sent',
        widget=StringWidget(
            label='Sent folder',
            label_msgid='BlueWebMail_label_imapSentFolder',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    StringField(
        name='imapDraftsFolder',
        default='INBOX.Drafts',
        widget=StringWidget(
            label='Drafts folder',
            label_msgid='BlueWebMail_label_imapDraftsFolder',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    BooleanField(
        name='imapShowPrefixOption',
        default=False,
        widget=BooleanWidget(
            label="Show prefix option",
            label_msgid='BlueWebMail_label_imapShowPrefixOption',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    BooleanField(
        name='imapDefaultSubOfInbox',
        default=True,
        widget=BooleanWidget(
            label="Default sub of inbox",
            label_msgid='BlueWebMail_label_imapDefaultSubOfInbox',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    BooleanField(
        name='imapShowContainSubfoldersOption',
        default=False,
        widget=BooleanWidget(
            label="Show contain subfolders option",
            label_msgid='BlueWebMail_label_imapShowContainSubfoldersOption',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    StringField(
        name='imapDelimiter',
        default='detect',
        widget=StringWidget(
            label='Delimiter',
            label_msgid='BlueWebMail_label_imapDelimiter',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    StringField(
        name='imapListPattern',
        default='',
        widget=StringWidget(
            label='Pattern for the IMAP list call',
            label_msgid='BlueWebMail_label_imapListPattern',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    BooleanField(
        name='imapDeleteFolder',
        default=False,
        widget=BooleanWidget(
            label="Delete folder",
            label_msgid='BlueWebMail_label_imapDeleteFolder',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    BooleanField(
        name='imapForceUsernameLowercase',
        default=False,
        widget=BooleanWidget(
            label="Force username lowercase",
            label_msgid='BlueWebMail_label_imapForceUsernameLowercase',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    StringField(
        name='folderCreationEncoding',
        vocabulary='getFolderCreationEncodingVocab',
        default='utf-8',
        widget=SelectionWidget(
            label='Allowed charset for folder names',
            label_msgid='BlueWebMail_label_folderCreationEncoding',
            i18n_domain='BlueWebMail',
        ),
    ),
    
    LinesField(
        name='groupMessageLimit',
        widget=LinesWidget(
            label='Limit allowed Number of Receivers of a Message for Groups, \
                   widcards are allowed, order is important, format: groupId:10',
            label_msgid='BlueWebMail_label_limit_allowed_messages_for_groups',
            i18n_domain='BlueWebMail',
        ),
    ),
),
)

WebMailTool_schema = BaseFolderSchema.copy() + \
    schema.copy()

class WebMailTool(UniqueObject, ActionProviderBase, BaseContent):
    """
    """
    security = ClassSecurityInfo()
    __implements__ = (ActionProviderBase.__implements__,
                      getattr(BaseContent,'__implements__',()),)
    
    manage_options = ActionProviderBase.manage_options \
                     + BaseContent.manage_options

    _actions_form = DTMLFile('www/editToolsActions',
                             os_path.join(package_home(globals())))

    archetype_name = 'WebMailTool'

    meta_type = 'WebMailTool'
    portal_type = 'WebMailTool'
    allowed_content_types = []
    filter_content_types = 1
    global_allow = 0
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "WebMailTool"
    typeDescMsgId = 'description_edit_webmailtool'
    toolicon = 'webmailtool_icon.png'
    
    actions =  (
       {'action': "string:${object_url}/base_view",
        'category': "object",
        'id': 'view',
        'name': 'view',
        'permissions': ("View",),
        'condition': 'python:1'
       },
    )

    _at_rename_after_creation = True

    schema = WebMailTool_schema
    
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_webmailtool')
        self.setTitle('WebMailTool')

    def at_post_edit_script(self):
        self.unindexObject()
    
    security.declareProtected( ManagePortal, 'manage_editActionsForm' )
    def manage_editActionsForm( self, REQUEST, manage_tabs_message=None ):
        """ Show the 'Actions' management tab.
        """
        
        zmiactions = [ ai.getMapping() for ai in self.listActions() ]
        # possible_permissions is in AccessControl.Role.RoleManager.
        pp = self.possible_permissions()
        return self._actions_form( self
                                 , REQUEST
                                 , zmiactions=zmiactions
                                 , possible_permissions=pp
                                 , management_view='Actions'
                                 , manage_tabs_message=manage_tabs_message
                                 )
    
    security.declarePrivate('_getPresetValueOrCustom')
    def _getPresetValueOrCustom(self, key, value):
        type = self.imapServerType
        if type == 'custom':
            ret = value
        else:
            ret = IMAPSERVER_PRESETS[type][key]
        return ret
    
    def _boolifyValue(self, value):
        if value == '0':
            return False
        return True
    
    security.declarePrivate('setImapDefaultFolderPrefix')
    def setImapFolderPrefix(self, value):
        value = value.strip()
        value = self._getPresetValueOrCustom('folder_prefix', value)
        self.imapFolderPrefix = value
    
    security.declarePrivate('setImapTrashFolder')
    def setImapTrashFolder(self, value):
        value = value.strip()
        value = self._getPresetValueOrCustom('trash_folder', value)
        self.imapTrashFolder = value
    
    security.declarePrivate('setImapSentFolder')
    def setImapSentFolder(self, value):
        value = value.strip()
        value = self._getPresetValueOrCustom('sent_folder', value)
        self.imapSentFolder = value
    
    security.declarePrivate('setImapDraftsFolder')
    def setImapDraftsFolder(self, value):
        value = value.strip()
        value = self._getPresetValueOrCustom('draft_folder', value)
        self.imapDraftsFolder = value
    
    security.declarePrivate('setImapShowPrefixOption')
    def setImapShowPrefixOption(self, value):
        value = self._boolifyValue(value)
        value = self._getPresetValueOrCustom('show_prefix_option', value)
        self.imapShowPrefixOption = value
    
    security.declarePrivate('setImapDefaultSubOfInbox')
    def setImapDefaultSubOfInbox(self, value):
        value = self._boolifyValue(value)
        value = self._getPresetValueOrCustom('default_sub_of_inbox', value)
        self.imapDefaultSubOfInbox = value
    
    security.declarePrivate('setImapShowContainSubfoldersOption')
    def setImapShowContainSubfoldersOption(self, value):
        value = self._boolifyValue(value)
        value = self._getPresetValueOrCustom('show_contain_subfolders_option',
                                             value)
        self.imapShowContainSubfoldersOption = value
    
    security.declarePrivate('setImapOptionalDelimiter')
    def setImapDelimiter(self, value):
        value = value.strip()
        value = self._getPresetValueOrCustom('delimiter', value)
        self.imapDelimiter = value
    
    security.declarePrivate('setImapDeleteFolder')
    def setImapDeleteFolder(self, value):
        value = self._boolifyValue(value)
        value = self._getPresetValueOrCustom('delete_folder', value)
        self.imapDeleteFolder = value
    
    security.declarePrivate('setImapForceUsernameLowercase')
    def setImapForceUsernameLowercase(self, value):
        value = self._boolifyValue(value)
        value = self._getPresetValueOrCustom('force_username_lowercase', value)
        self.imapForceUsernameLowercase = value
    
    security.declarePrivate('setFolderCreationEncoding')
    def setFolderCreationEncoding(self, value):
        value = value.strip()
        value = self._getPresetValueOrCustom('folder_creation_encoding', value)
        self.folderCreationEncoding = value
    
    security.declarePrivate('getImapServerTypesVocab')
    def getImapServerTypesVocab(self):
        ordered_keys = [
            'default', 'courier', 'cyrus', 'bincimap1.2.13', 'bincimap1.3.4',
            'dovecot', 'exchange', 'hmailserver', 'macosx', 'mercury32', 'uw',
        ]
        ret = [(key, IMAPSERVER_TYPES[key]) for key in ordered_keys]
        ret += [('custom', 'Custom Settings')]
        return ret
    
    security.declarePrivate('getFolderCreationEncodingVocab')
    def getFolderCreationEncodingVocab(self):
        return [
            ('utf-8', 'UTF 8'),
            ('ascii', 'ASCII'),
            ('iso-8859-1', 'ISO 8859-1'),
        ]
    
    security.declarePrivate('getCurrentUsersProperty')
    def getCurrentUsersProperty(self, property, default=None):
        mt = getToolByName(self, 'portal_membership')
        user = mt.getAuthenticatedMember()
        prop = user.getProperty(property, default)
        if not prop:
            imappw = default
        return prop
    

registerType(WebMailTool, PROJECTNAME)
