# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

from zope.interface import implements
from Products.Five import BrowserView

from Products.BlueWebMail.interfaces import IWebmailProperties
from Products.BlueWebMail.interfaces import IIMAPActionsView

from Products.BlueWebMail import keeper

class IMAPActionsView(BrowserView):
    """Concrete Implementation of IIMAPActionsView.
    """
    
    implements(IIMAPActionsView)
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.actions = False
        self.displaysections = []
        
    def initialize(self):
        ic = keeper.tool.getController(self.context, self.request, 10)
        self._mailbox = ic.mailbox
    
    def getActions(self):
        """Return the actions to show.
        TODO: other solution than ugly one with url parsing. """
        if self.actions is not False:
            return self.actions
        
        path = self.request.get('PATH_TRANSLATED')
        path = path[path.rfind('/') + 1:]
        props = IWebmailProperties(self.context)
        
        if path == 'create_message':
            self.displaysections.append('createActions')
            self.actions = props.actions['BlueWebMail_Create']
            return self.actions
        
        self.displaysections.append('globalActions')
        actions = props.actions['BlueWebMail_Global']
        
        if path == 'view' \
          or path == 'webmail_view' \
          or path == self.context.id:
            self.displaysections.append('folderActions')
            actions = actions + props.actions['BlueWebMail_Folder']
        if path == 'message_view':
            self.displaysections.append('messageActions')
            actions = actions + props.actions['BlueWebMail_Message']
        
        self.displaysections.append('movementActions')    
        actions = actions + props.actions['BlueWebMail_Movement']
        self.actions = actions
        return self.actions
    
    def getDisplaySections(self):
        """Return a list containing which sections to display.
        """
        if self.actions is False:
            self.getActions()
        return self.displaysections
    
    def getIMAPFolders(self):
        """Return the IMAP Folder names contained in this MailBox as list of
        tuples.
        """
        ret = []
        folders = self._mailbox.listFolders()
        prefix = self._mailbox.folder_prefix
        delim = self._mailbox.delimiter
        inbox = self._mailbox.default_folders['inbox']
        
        for folder in folders:
            name = folder[1]
            readable = folder[1]
            if readable != inbox:
                readable = readable[len(prefix):]
            readable = name.replace(delim, '/')
            readable = readable.replace('_', ' ')  
            ret.append((name, readable))
        
        return ret
 