# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

from zope.interface import implements
from Products.Five import BrowserView

from Products.BlueWebMail.interfaces import IWysiwygSupportView
from Products.BlueWebMail.interfaces import IWebmailStatusMessage
from Products.BlueWebMail.interfaces import IWebmailJavascriptLoader
from Products.BlueWebMail.WebMailBox import WebMailBox
from Products.BlueWebMail.util import getWebmailMessage


class WebmailStatusMessage(BrowserView):
    """See interfaces.browser.IWebmailStatusMessage for details.
    """
    
    implements(IWebmailStatusMessage)
    
    def getMessage(self):
        """Return the current webmail status message if there is one.
        """
        return getWebmailMessage(self.request)


class WebmailJavascriptLoader(BrowserView):
    """See interfaces.browser.IWebmailJavascriptLoader for details
    """
    
    implements(IWebmailJavascriptLoader)
    
    def isWebmailContext(self):
        if isinstance(self.context, WebMailBox):
            return True
        return False


class WysiwygSupportView(BrowserView):
    """See interfaces.IWysiwygSupportView for details.
    """
    
    implements(IWysiwygSupportView)
    
    FIXED = ['firefox']
    ALLOWED = ['mozilla', 'gecko', 'msie']
    FORBIDDEN = ['opera', 'apple', 'safari', 'macintosh']
    
    def loadCssAndJavacripts(self):
        context = self.context
        request = self.request
        
        if not isinstance(context, WebMailBox):
            return context.kupu_library_tool.isKupuEnabled(REQUEST=request)
        
        # special cases of switching between modes
        if request.get('switchToHtml', None):
            return True
        
        if request.get('switchToText', None):
            return False
        
        # default call
        return self.checkCreateFormat('html')
        
    def checkCreateFormat(self, format):
        context = self.context
        
        if not isinstance(context, WebMailBox):
            return False
        
        format = format.lower()
        createFormat = context.getCreateFormat().lower()
        
        if format not in ['text', 'html']:
            return False
        
        if self._browserValid() and createFormat == format:
            return True 
        
        return False
    
    def displayHtmlButton(self):
        return self.checkCreateFormat('text')
        
    def displayTextButton(self):
        return self.checkCreateFormat('html')
    
    def _browserValid(self):
        request = self.request

        useragent = request.get('HTTP_USER_AGENT', '').lower()
        
        # hope this hack fits its needs
        for fixed in self.FIXED:
            if useragent.find(fixed) != -1:
                return True
        
        for allowed in self.ALLOWED:
            if useragent.find(allowed) != -1:
                for forbidden in self.FORBIDDEN:
                    if useragent.find(forbidden) != -1:
                        return False
                return True
        return False
