# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

from zope.interface import implements
from Products.Five import BrowserView

from Products.BlueWebMail.interfaces import IWebmailProperties
from Products.BlueWebMail.interfaces import IIMAPFolderView

from Products.BlueWebMail.util import decodeHeader
from Products.BlueWebMail import keeper

class IMAPFolderView(BrowserView):
    """Concrete Implementation of IWebMailIndexView.
    """
    
    implements(IIMAPFolderView)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.foldername = False
        self.messageCount = None
        self.pagedefs = False
        self.currentpage = False
        self.previouspage = False
        self.nextpage = False
        
    def initialize(self):
        ic = keeper.tool.getController(self.context, self.request, 10)
        self._mailbox = ic.mailbox
        props = IWebmailProperties(self.context)
        self.displaymaxmessages = props.max_messages
    
    def skip(self):
        return self.request.get('skip_remaining', False)
            
    def getPageDefs(self):
        """Return page definition params. internal use only.
        """
        if self.pagedefs is False:
            count = self.getCount()
            range = self.displaymaxmessages
            start = self.request.get('start', '1')
            end = self.request.get('end', range)
            self.pagedefs = dict()
            self.pagedefs['c'] = int(count)
            self.pagedefs['r'] = int(range)
            self.pagedefs['s'] = int(start)
            self.pagedefs['e'] = int(end)
        
        c = self.pagedefs['c']
        r = self.pagedefs['r']
        s = self.pagedefs['s']
        e = self.pagedefs['e']
        return c, r, s, e
    
    def getPager(self):
        """Return pager.
        """
        ret = []
        c, r, s, e = self.getPageDefs()
        p = c / r
        if not c % r == 0:
            p += 1
        for i in range(p):
            ret.append({'start': str((i + 1) * r - r + 1),
                        'end': str((i + 1) * r > c and c or (i + 1) * r),
                        'current': (i + 1) * r - r + 1 == s and True or False,
                        'page': str(i + 1)})
        
        return ret
    
    def getCurrentPage(self):
        """Return current page start and end.
        """
        if self.currentpage is False:
            count, range, start, end = self.getPageDefs()
            if end >= count:
                end = count
            self.currentpage = {'start': str(start), 'end': str(end)}
        return self.currentpage
        
    def getNextPage(self):
        """Return next page start and end.
        """
        if self.nextpage is False:
            count, range, start, end = self.getPageDefs()
            start = end + 1
            end = end + range
            if end >= count + range:
                return {'start': None, 'end': None}
            if end >= count:
                end = count
            self.nextpage = {'start': str(start), 'end': str(end)}
        return self.nextpage
        
    def getPrevPage(self):
        """Return prev page start and end.
        """
        if self.previouspage is False:
            count, range, start, end = self.getPageDefs()
            end = start - 1
            start = start - (range)
            if start < 1:
                return {'start': None, 'end': None}
            self.previouspage = {'start': str(start), 'end': str(end)}
        return self.previouspage
        
    def getCount(self):
        """Return the number of message contained in given Folder.
        """
        if not self.messageCount:
            self.messageCount = self._getFolder().getMessageCount()
        return self.messageCount
                
    def getFolderName(self):
        """Return the folderName of the current Folder.
        """
        if self.foldername is False:
            name = self._getFolder().name
            delim = self._mailbox.delimiter
            readable = name.replace(delim, '/')
            readable = readable.replace('_', ' ')
            self.foldername = {'readable': readable,
                               'real': name }
        return self.foldername
        
    def getIMAPHeaders(self):
        """Return the Message Headers contained in given Folder.
        
        TODO: all this header decoding and flag verification things should be
        done at IMAPFolder. When this is done, also cachevary must be changed!
        """
        s = self.request.get('start', '1')
        e = self.request.get('end', self.displaymaxmessages)
        c = self.getCount()
        
        folder = self._getFolder()
        headers = folder.listHeaders(sortmail=self.request.get('sort', ''),
                                     start=0,
                                     end=int(c))
        
        if self.request.get('way', '').strip() == 'desc':
            headers.reverse()
            
        headers = headers[int(s)-1:int(e)]
        
        ret = []
        for header in headers:
            h = {}
            h['id'] = header['imapId']
            
            if header['flags'].find('\Deleted') > -1:
                h['deleted'] = 'deleted'
            else:
                h['deleted'] = ''
            
            if header['flags'].find('\Answered') > -1:
                h['seen'] = 'answered'
            elif header['flags'].find('\Seen') > -1:
                h['seen'] = 'seen'
            else:
                h['seen'] = 'new'
            
            if header['message'].get('Content-Type', '').find('multipart/mixed') > -1:
                h['att'] = 'att'
            else:
                h['att'] = 'noAtt'
            
            xprio = header['message'].get('X-Priority')
            if xprio is not None:
                try:
                    xprio = int(xprio)
                    if xprio < 3:
                        h['prio'] = 'prioHi'
                    elif xprio > 3:
                        h['prio'] = 'prioLo'
                    else:
                        h['prio'] = 'prioNo'
                except:
                    if xprio.find('Normal') > -1:
                        h['prio'] = 'prioNo'
                    else:
                        h['prio'] = xprio
            else:
                h['prio'] = None
            
            fromline = decodeHeader(header['message'].get('From', ''))
            if len(fromline) > 40:
                fromline = '%s %s' % (fromline[0:35], '...')
            h['from'] = fromline
            
            subject = decodeHeader(header['message'].get('Subject', ''))
            if len(subject) > 40:
                subject = '%s %s' % (subject[0:35], '...')
            h['subject'] = subject
            
            date = header['message'].get('Date', '')
            if date.find('+') > -1:
                h['date'] = date[0:date.find('+')].strip()
            elif date.find('-') > -1:
                h['date'] = date[0:date.find('-')].strip()
            else:
                h['date'] = date
                
            ret.append(h)
            
        return ret
    
    def getPagerUrl(self, page):
        """Return pager url.
        """
        folderName, sort, way = self._getControlParams(False)
        start = page['start']
        end = page['end']
        
        baseurl = self.context.absolute_url()
        query = '?folder=%s&start=%s&end=%s&sort=%s&way=%s'
        query = query % (folderName, start, end, sort, way)
        
        url = '%s/%s' % (baseurl, query)
        
        return url
    
    def getListingHeaderUrl(self, sorton):
        """Return listing header url
        """
        folderName, start, end, sort, way = self._getControlParams(True)
        
        baseurl = self.context.absolute_url()
        query = '?folder=%s&start=%s&end=%s&sort=%s&way=%s'
        
        # special case DATE
        if way == '' and sorton == 'DATE':
            sortway = 'desc'
        # other cases
        else:
            if way == '':
                way = 'desc'
            sortway = (way == 'asc' and sort == sorton) and 'desc' or 'asc'
        
        query = query % (folderName, start, end, sorton, sortway)
        url = '%s/%s' % (baseurl, query)
        
        return url
    
    def getMessageUrl(self, imapId):
        """Return the url to the imap message.
        
        There is a special case if the current folder is Drafts. Then we want
        to go to the create mask filled in the values of the mail.
        """
        folderName, start, end, sort, way = self._getControlParams(True)
        url = self.context.absolute_url()
        defaults = self._mailbox.default_folders
        
        query = '?folder=%s&imapId=%s&sort=%s&way=%s&start=%s&end=%s'
        query = query % (folderName, imapId, sort, way, start, end)
        target = '/message_view'
        if folderName == defaults['drafts']:
            target = '/create_message'
            query = '%s&createAction=Draft' % query
        ret = '%s%s%s' % (url, target, query)
        
        return ret
    
    def _getControlParams(self, includeCurrentPageDefs):
        req = self.request
        fname = self.getFolderName()['real']
        sort = req.get('sort', '')
        way = req.get('way', '')
        
        if includeCurrentPageDefs:
            current = self.getCurrentPage()
            start = current['start']
            end = current['end']
            return fname, start, end, sort, way
        
        return fname, sort, way
    
    def _getFolder(self):
        folder = self.request.get('folder',
                                  self._mailbox.default_folders['inbox'])
        return self._mailbox.getFolder(folder)
    