# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

import types

from zope.interface import implements
from Products.Five import BrowserView

from email.Utils import getaddresses

from Products.CMFPlone.i18nl10n import utranslate

from Products.BlueWebMail.interfaces import IIMAPMessageView
from Products.BlueWebMail.interfaces import IIMAPCreateMessageView
from Products.BlueWebMail.interfaces import IWebmailProperties

from Products.BlueWebMail.util import decodeHeader
from Products.BlueWebMail import keeper
from Products.BlueWebMail.messageparthandler import MessagePartHandler
from Products.BlueWebMail.browser import WysiwygSupportView


class IMAPMessageView(BrowserView):
    """Concrete Implementation of IIMAPMessageView.
    """
    
    implements(IIMAPMessageView)
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
    
    def initialize(self):
        context = self.context
        request = self.request
        ic = keeper.tool.getController(context, request, 10)
        self._mailbox = ic.mailbox
        self._inbox = self._mailbox.default_folders['inbox']
        self._delim = self._mailbox.delimiter
        
        id = request.get('imapId', '').strip()
        if id == '':
            folder = self.getFolderName()['real']
            sort = request.get('sort', '')
            way = request.get('way', '')
            start = request.get('start', '')
            end = request.get('end', '')
            url = '%s%s%s%s%s%s' % (context.absolute_url(),
                                    '%s%s' % ('?sort=', sort),
                                    '%s%s' % ('&way=', way),
                                    '%s%s' % ('&start=', start),
                                    '%s%s' % ('&end=', end),
                                    '%s%s' % ('&folder=', folder))
            request.RESPONSE.redirect(url)
        
        if type(id) is types.StringType:
            folder = self.request.get('folder', self._inbox)
            message = self._mailbox.getFolder(folder).getMessages([id])
            self.message = message[0]
        else:
            # there are more than one message ids, thats invalid.
            # TODO: error handling.
            self.message = None
    
    def skip(self):
        return self.request.get('skip_remaining', False)
    
    def showMessagePartInformation(self):
        return IWebmailProperties(self.context).message_part_information
    
    def getFolderName(self):
        """Return the folderName of the current Folder.
        """
        folder = self.request.get('folder', self._inbox)
        name = self._mailbox.getFolder(folder).name
        return {'readable': name.replace(self._delim, '/'),
                'real': name }
                
    def getTo(self):
        """Return the receiver.
        """
        return decodeHeader(self.message.getHeader('To'))
        
    def getFrom(self):
        """Return the senders.
        """
        return decodeHeader(self.message.getHeader('From'))
    
    def getCc(self):
        """Return the Cc's.
        """
        return decodeHeader(self.message.getHeader('Cc'))
    
    def getSubject(self):
        """Return the subject.
        """
        return decodeHeader(self.message.getHeader('Subject'))
    
    def getDate(self):
        """Return the date.
        """
        return self.message.getHeader('Date')
    
    def getParts(self):
        """Return the message body.
        """
        mph = MessagePartHandler(self.context)
        return mph.getMessageParts(self.message)


class IMAPCreateMessageView(BrowserView):
    """The Create Message View Class.
    """
    
    implements(IIMAPCreateMessageView)
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.body = ''
        self.forwardattachment = None
        self.draftattachments = []
            
    def initialize(self):
        ic = keeper.tool.getController(self.context, self.request, 10)
        self._mailbox = ic.mailbox
        self._inbox = self._mailbox.default_folders['inbox']
        self._delim = self._mailbox.delimiter
        createAction = self.request.get('createAction', '')
        self.createAction = createAction
        
        # ugly workaround since redirect does not work immediatly 
        self.saveaction = False
        self.sendaction = False
        if self.request.get('save', None) is not None:
            self.saveaction = True
            return
        if self.request.get('send', None) is not None:
            self.sendaction = True
            return
        
        if not self.saveaction \
          and not self.sendaction:
            self._readMessage()
            self.wysiwygSupport = WysiwygSupportView(self.context, self.request)
        
        if createAction == 'Reply' \
          or createAction == 'Reply All' \
          or createAction == 'Draft':
            self._handleParts()
        
        if createAction == 'Forward':
            fwdmessage = dict()
            fwdmessage['id'] = self.request.get('imapId', '').strip()
            fwdmessage['folder'] = self.request.get('folder', '').strip()
            fwdmessage['name'] = 'Forwarded message'
            fwdmessage['filename'] = self.message.getHeader('Subject')
            self.forwardattachment = fwdmessage 
    
    def skip(self):
        return self.request.get('skip_remaining', False)           
        
    def getFrom(self):
        """Return a 2-tuple containing email address and Real Name.
        """
        props = IWebmailProperties(self.context)
        return (props.email_address, props.real_name)
    
    def getTo(self):
        """Return the value of the to field for creating a message.
        """
        if self.saveaction or self.sendaction:
            return ''
        
        createAction = self.createAction
        if createAction == 'Draft':
            return self.message.getHeader('To')
        
        if createAction == 'Reply' or createAction == 'Reply All':
            return self.message.getHeader('From')
        
        return ''
        
    def getCc(self):
        """Return the value of the cc field for creating a message.
        """
        if self.saveaction or self.sendaction:
            return ''
        
        createAction = self.createAction
        if createAction == 'Draft':
            return self.message.getHeader('Cc')
        
        if createAction == 'Reply All':
            props = IWebmailProperties(self.context)
            myaddress = props.email_address
            to = self.message.message.get_all('to', [])
            cc = self.message.message.get_all('cc', [])
            all = getaddresses(to + cc)
            
            ret = ''
            for address in all:
                if address[1] != myaddress:
                    real = address[0]
                    email = address[1]
                    if real != '':
                        addr = '%s <%s>' % (real, email)
                    else:
                        addr = email
                    ret = '%s %s, ' % (ret, addr)
                    ret = ret.strip()
            
            return ret
        
        return ''
    
    def getBcc(self):
        """ """
        if self.saveaction or self.sendaction:
            return ''
        
        createAction = self.createAction
        if createAction == 'Draft':
            return self.message.getHeader('Bcc')
        
        return ''
    
    def getSubject(self):
        """Return the value of the subject field for creating a message.
        """
        if self.saveaction or self.sendaction:
            return ''
        
        createAction = self.createAction
        if createAction == 'Draft':
            return decodeHeader(self.message.getHeader('Subject'))
        
        if createAction == 'Reply' \
          or createAction == 'Reply All' \
          or createAction == 'Forward':
            prefix = 'Re:'
            if createAction == 'Forward':
                prefix = 'Fwd:'
            
            subject = decodeHeader(self.message.getHeader('Subject'))
            return '%s %s' % (prefix, subject)
        
        return ''
    
    def getBodyText(self):
        """Return the body text for creating a message.
        
        TODO:
            * some troubleshootingwigh transform that message is displayed
              properly when switching between modes
            * do better plain text formatting when replying to a message
        """
        if self.saveaction or self.sendaction:
            return ''
        
        isHtml = self.wysiwygSupport.checkCreateFormat('html')
        createAction = self.createAction
        reqbody = self.request.get('messageText', '')
        if createAction == 'New':
            return self._transformBodyText(isHtml, reqbody)
                
        if createAction == 'Draft':
            if reqbody:
                return self._transformBodyText(isHtml, reqbody)
            return self.body
        
        if createAction == 'Reply' \
          or createAction == 'Reply All':
            if reqbody:
                return self._transformBodyText(isHtml, reqbody)
            linefeed = isHtml and '<br /><br />' or '\n\n'
            wrote = utranslate('bluewebmail', u'wrote:', context=self.context)
            fromaddr = self.message.getHeader('From').encode('utf-8')
            return '%s %s%s%s' % (fromaddr, wrote.encode('utf-8'),
                                  linefeed, self.body)
        
        if createAction == 'Forward':
            if reqbody:
                return self._transformBodyText(isHtml, reqbody)
            fromheader = self.message.getHeader('From')
            fromheader = fromheader and fromheader or ''
            dateheader = self.message.getHeader('Date')
            dateheader = dateheader and dateheader or ''
            linefeed1 = isHtml and '<br />' or '\n'
            linefeed2 = isHtml and '<br /><br />' or '\n\n'
            
            original_message = utranslate('bluewebmail',
                                          u'Original message:',
                                          context=self.context)
            
            return '%s\n%s%s%s%s' % (original_message, fromheader, linefeed1,
                                     dateheader, linefeed2)
    
    def getForwardAttachment(self):
        createAction = self.createAction
        if createAction == 'Forward':
            return self.forwardattachment
        return None
    
    def getDraftAttachments(self):
        createAction = self.createAction
        if createAction == 'Draft':
            return self.draftattachments
        return []
    
    def _readMessage(self):
        """Read the message from the server.
        """
        id = self.request.get('imapId', '').strip()
        
        # do some redirecting if there is no imapId
        
        if type(id) is types.StringType:
            folder = self.request.get('folder', self._inbox)
            message = self._mailbox.getFolder(folder).getMessages([id])
            self.message = message[0]
        else:
            # there are more than one message ids, thats invalid.
            # TODO: error handling.
            self.message = None
    
    def _handleParts(self):
        """Handle the message parts.
        """
        mph = MessagePartHandler(self.context)
        parts = mph.getMessageParts(self.message, True, self.wysiwygSupport)
        req = self.request
        
        createAction = self.createAction
        for part in parts:
            if not part['isAttachment']:
                data = part['data']
                if not data:
                    data = ''
                self.body += data
            else:
                if createAction == 'Draft':
                    attachment = dict()
                    attachment['id'] = req.get('imapId', '').strip()
                    attachment['folder'] = req.get('folder', '').strip()
                    attachment['name'] = part['name']
                    attachment['filename'] = part['filename']
                    self.draftattachments.append(attachment)
    
    def _transformBodyText(self, isHtml, body):
        transform = self.context.portal_transforms
        if isHtml:
            return transform.convert('text_to_html', body).getData()
        else:
            return transform.convert('html_to_text', body).getData()
