# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

from zope.interface import implements
from Products.Five import BrowserView

from Products.BlueWebMail.interfaces import IWebmailProperties
from Products.BlueWebMail.interfaces import IWebMailOverviewPortletView
from Products.BlueWebMail.interfaces import IWebMailPortletView

from Products.BlueWebMail import keeper
from Products.BlueWebMail import control
from Products.BlueWebMail.WebMailBox import WebMailBox

class WebMailOverviewPortletView(BrowserView):
    """View definitions for the webmail overview portlet.
    """
    
    implements(IWebMailOverviewPortletView)
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self._mailbox = None
        self._mailboxavailable = None
        self._mailboxurl = ''
        
    def initialize(self):
        mb = control.getMailboxForAuthenticatedUser(self.context)
        if mb is None:
            self._mailbox = None
            self._mailboxavailable = False
        else:
            ic = keeper.tool.getController(mb[0], self.request, 10)
            self._mailbox = ic.mailbox
            self._mailboxavailable = True
            self._mailboxurl = mb[1]
    
    def display(self):
        """Wether to display the Portlet or not.
        """
        if isinstance(self.context, WebMailBox):
            return False
        return True
    
    def isMailboxAvailable(self):
        """Return True if a WebMailBox object is available for the current
        authenticated user.
        """
        return self._mailboxavailable
    
    def getMailboxUrl(self):
        """Return the absolute ULR to the WebMailBox object for the current
        authenticated user.
        """
        return self._mailboxurl
        
    def getNewMessagesCount(self):
        """Return the count of new messages contained in the inbox.
        """
        inbox = self._mailbox.default_folders['inbox']
        return self._mailbox.getFolder(inbox).getUnreadCount()

class WebMailPortletView(BrowserView):
    """View class For the IMAP Folder listing Portlet.
    """
    
    implements(IWebMailPortletView)
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self._mailbox = None
        self._tree = None
        self._selected_folder = None
        
    def initialize(self):
        ic = keeper.tool.getController(self.context, self.request, 10)
        self._mailbox = ic.mailbox
    
    def skip(self):
        return self.request.get('skip_remaining', False)
    
    def getRealname(self):
        """Return the realname of the user.
        """
        return IWebmailProperties(self.context).real_name
    
    def getSelectedFolder(self):
        """Return the currently selected folder
        """
        if self._selected_folder is not None:
            return self._selected_folder
        
        selected_folder = ''
        tree = self.getImapFolders()
        for node in tree:
            if node['selected'] is True:
                selected_folder = node['folder']
                break
        
        self._selected_folder = selected_folder
        return selected_folder
    
    def getSubfolders(self):
        """Return subfolders of current selected folder. used for the delete
        folder selection. default folders are skipped.
        """
        tree = self.getImapFolders()
        selectedfolder = self.getSelectedFolder()
        prefix = self._mailbox.folder_prefix
        inbox = self._mailbox.default_folders['inbox']
        default_sub = self._mailbox.default_sub_of_inbox
        defaults = self._mailbox.default_folders
        defaults = [defaults[key] for key in defaults.keys()]
        
        # TODO: something with the default sub of inbox, no imap server
        # used yet with this behaviour
        
        subfolders = []
        for node in tree:
            folder = node['folder']
            
            if selectedfolder == inbox:
                if not folder in defaults:
                    subfolder = dict()
                    start = len(prefix)
                    subfolder['name'] = folder[start:].replace('_', ' ')
                    subfolder['folder'] = folder
                    subfolders.append(subfolder)
            elif folder.startswith(selectedfolder):
                if not folder == selectedfolder \
                  and not folder in defaults:
                    subfolder = dict()
                    start = len(selectedfolder) + 1
                    subfolder['name'] = folder[start:].replace('_', ' ')
                    subfolder['folder'] = folder
                    subfolders.append(subfolder)
        return subfolders
    
    def getImapFolders(self):
        """Return a tree representing structure for further displaying in the
        template.
        """
        if self._tree is None:
            inbox = self._mailbox.default_folders['inbox']
            current = self.request.get('folder', inbox).strip()
            self._tree = self._mailbox.getFolderTree(current)
        return self._tree
 