# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

from zope.interface import Interface

class IWebmailStatusMessage(Interface):
    """Provide a seperate status message for webmail.
    """
    
    def getMessage():
        """Return the current webmail status message if there is one.
        """


class IWebmailJavascriptLoader(Interface):
    """Object providing context information for javascript loading.
    """
    
    def isWebmailContext():
        """Return wether context is webmail or not.
        """


class IWysiwygSupportView(Interface):
    """This interface implementing object is used to determine if wysiwyg
    editor support is configured and enabled.
    """
    
    def loadCssAndJavacripts():
        """This function is used by portal_css and portal_javascript to
        determine wether the kupu related files have to be loaded or not.
        
        By installing the webmailer, the existing conditions are altered by
        a call to this function.
        """
    
    def checkCreateFormat(format):
        """Check for the current preferred create format.
        
        Expect either 'text' or 'html' in format param.
        
        First check if WebMailBox is the callers context, then if the browser
        used for the current request is able to display wysiwyg support.
        
        Finally check the given format and return wether it is the current
        preferred create format or not.
        """
    
    def displayHtmlButton():
        """Return wether to display the 'switch to html mode' button or not.
        """
        
    def displayTextButton():
        """Return wether to display the 'switch to text mode' button or not.
        """
        

class IWebMailOverviewPortletView(Interface):
    """View definitions for the webmail overview portlet.
    """
    
    def initialize():
        """Do initialization of the object after the real __init__ was called
        because we need things that have not been initialized at this time.
        """
    
    def display():
        """Wether to display the Portlet or not.
        """
    
    def isMailboxAvailable():
        """Return True if a WebMailBox object is available for the current
        authenticated user.
        """
    
    def getMailboxUrl():
        """Return the absolute ULR to the WebMailBox object for the current
        authenticated user.
        """
    
    def getNewMessagesCount():
        """Return the count of new messages contained in the inbox.
        """

    
class IWebMailPortletView(Interface):
    """View definitions For the IMAP Folder listing Portlet.
    """
    
    def initialize():
        """Do initialization of the object after the real __init__ was called
        because we need things that have not been initialized at this time.
        """
    
    def skip():
        """This is a hack to avoid multiple template rendereing after a redirect
        is called since redirecting out of browser views does not behave as
        expected and return immediately.
        """
    
    def getImapFolders():
        """Return the IMAP Folder names contained in this MailBox.
        
        promise to return a list of dicts in this form:
        [
            {
                'level': '1',
                'name': 'Drafts',
                'folder': 'INBOX.Drafts',
                'selected': True,
                'unread': 3,
            }
        ]
        """
        
    def getRealname():
        """Return the realname of the user.
        """
    
    def getSelectedFolder():
        """Return the currently selected folder
        """
        
    def getSubfolders():
        """Return subfolders of current selected folder
        """


class IIMAPActionsView(Interface):
    """The Actions View definitions.
    """
    
    def initialize():
        """Do initialization of the object after the real __init__ was called
        because we need things that have not been initialized at this time.
        """
    
    def getActions():
        """Return the actions to show.
        """
        
    def getDisplaySections():
        """Return a list containing which sections to display.
        """
        
    def getIMAPFolders():
        """Return the IMAP Folder names contained in this MailBox.
        """


class IIMAPFolderView(Interface):
    """The Folder View definitions.
    """
    
    def initialize():
        """Do initialization of the object after the real __init__ was called
        because we need things that have not been initialized at this time.
        """
    
    def skip():
        """This is a hack to avoid multiple template rendereing after a redirect
        is called since redirecting out of browser views does not behave as
        expected and return immediately.
        """
    
    def getCurrentPage():
        """Return current page start and end.
        """
        
    def getPager():
        """Return pager.
        """
    
    def getNextPage():
        """Return next page start and end.
        """
        
    def getPrevPage():
        """Return prev page start and end.
        """
        
    def getFolderName():
        """Return the folderName of the current Folder.
        """
        
    def getIMAPHeaders(folder):
        """Return the Message Headers contained in given Folder.
        """
    
    def getCount(folder):
        """Return the number of message contained in given Folder.
        """
    
    def getPagerUrl(page):
        """Return pager url.
        """
    
    def getListingHeaderUrl(sorton):
        """Return listinge header url.
        """
        
    def getMessageUrl(imapId):
        """Return the url to the imap message.
        
        There is a special case if the current folder is Drafts. Then we want
        to go to the create mask filled in the values of the mail.
        """
                    
    def finalize():
        """Finalize session.
        """
        
class IIMAPMessageView(Interface):
    """The Message View Definitions.
    """
    
    def initialize():
        """Do initialization of the object after the real __init__ was called
        because we need things that have not been initialized at this time.
        """
    
    def skip():
        """This is a hack to avoid multiple template rendereing after a redirect
        is called since redirecting out of browser views does not behave as
        expected and return immediately.
        """
    
    def showMessagePartInformation():
        """Return wether to show the message part information or not.
        """
    
    def getFolderName():
        """Return the folderName of the current Folder.
        """
    
    def getTo():
        """Return the receiver.
        """
    
    def getFrom():
        """Return the senders.
        """
        
    def getCc():
        """Return the Cc's.
        """
        
    def getSubject():
        """Return the subject.
        """
        
    def getDate():
        """Return the date.
        """
        
    def getParts():
        """Return the message body.
        """
        
class IIMAPCreateMessageView(Interface):
    """The Create Message View Class.
    """
    
    def initialize():
        """Do initialization of the object after the real __init__ was called
        because we need things that have not been initialized at this time.
        """
    
    def skip():
        """This is a hack to avoid multiple template rendereing after a redirect
        is called since redirecting out of browser views does not behave as
        expected and return immediately.
        """
    
    def getFrom():
        """Return a 2-tuple containing email address and Real Name.
        """
    
    def getTo():
        """
        """
        
    def getCc():
        """
        """
        
    def getBcc():
        """
        """
    
    def getSubject():
        """
        """
    
    def getBodyText():
        """
        """
    
    def getForwardAttachment():
        """
        """
    
    def getDraftAttachments():
        """
        """ 