# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

from zope.interface import Interface

class IMailSendout(Interface):
    """API for sending E-Mails.
    
    Adapts a email.Message object. Do a lookup for the headers 'To', 'Cc' and
    'Bcc', remove the bcc header and send the message on calling send(). 
    """
    
    def initialize(data=None):
        """Initialize the Sendout object.
        
        Normally you do some things like lookind up the smtp server to use,
        the username and password for connecting to the smtp server and
        extracting the receivers from the message.
        
        Therefor you have the ability to otional give any data as argument
        if you need to have access f.e. to any other context providing needed
        data."""
    
    def send():
        """Send the E-Mail.
        
        This method finally sends the message.
        """

class IDispatchRuleSet(Interface):
    """API for restrictions on sending emails.
    
    You might want to restrict the number of allowed receipients of a message
    contained in a group, so this Interface describes how to do that. This
    DispatchRuleSet only applies on groups from personal addressbook.
    """
    
    def appliesToRuleSet(data=None):
        """Check the email.Message object and return True if message RuleSet
        applies to given message object or something that indicates the possible
        violation of defined rules.
        """
    
    def getCuttentLimit():
        """Return the limit for the current authenticated member.
        
        Needed for creating error messages.
        """
        
    def getCuttentCount():
        """Return the count of receivers contained in the message.
        
        Needed for creating error messages.
        """
        
class IReceiverResolver(Interface):
    """This Interface defines an adapter between a message object and a receiver
    provider.
    
    It takes the message as context and on calling resolve() it will resolve
    the receivers and return the modified message object.
    """
    
    def resolve(data=None):
        """Resolve the receivers.
        
        Read the message headers to, cc and bcc, iterate over its value(s) 
        and resolve its values in special cases like group mailing.
        
        Its possible to give additional data if you want to deal with something
        else than the message object.
        """
    
    def getInvalid():
        """Return the invalid receivers found and removed in this objects
        context.
        """


class IReceiverProvider(Interface):
    """This interface defines a mechanism to resolve receiver adresses.
    
    It Adapts an IReceiverRecord object and on calling process it resolves the
    record name to receiver adresses.
    """
    
    def process(data=None):
        """Resolve the record name to receiver adresses and write those
        adresses to the receiver record by calling setReceivers().
        
        Its possible to give additional data if you want to deal with something
        else than the receiverRecord object.
        """

class IReceiverRecord(Interface):
    """This Interface defines a record of receivers.
    """
    
    def getRecordName():
        """Return the name of this record
        """
    
    def setReceivers(receivers):
        """Expect a list of receivers as string.
        """
    
    def getReceivers():
        """Return this records receivers as list.
        """
 