# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Jens Klein <jens@bluedynamics.com>,
                Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

from OFS.Image import File
from interfaces import IWebmailProperties

from Products.BlueWebMail.util import decodeHeader

class MessagePartHandler(object):
    
    def __init__(self, context):
        self.context = context
        self.request = context.REQUEST
        
        #TODO: make media types configureable
        self.audiotypes = [
           'audio/mpeg',
           'audio/x-wav',
        ]
        self.videotypes = [
            'video/x-ms-wmv',
            'video/x-ms-asf',
            'video/mpeg',
            'video/x-msvideo',
            'video/quicktime',
        ]
    
    def getMessageParts(self, message, isCreation=False, wysiwygSupport=None):
        """Return the parts of the message body.
        """
        props = IWebmailProperties(self.context)
        preferredtypes = {
            'TEXT': 'text/plain',
            'HTML': 'text/html',
        }
        messagetype = message.getMessage().get_content_type()
        transform = self.context.portal_transforms
        parts = message.getParts()
        ret = []
        for part in parts:
            ret.append(self._resolveMessageParts(part, isCreation,
                                                 wysiwygSupport, props,
                                                 preferredtypes, messagetype,
                                                 transform))    
        return ret
    
    def _resolveMessageParts(self, part, isCreation, wysiwygSupport,
                             props, preferredtypes, messagetype, transform):
        """Function resolves recursive the given messageparts.
        """
        type = part.contenttype
        charset = part.charset
        isAttachment = part.attachment
        filename = part.filename
        name = part.name
        encoding = part.encoding
        data = None
        
        texttypes = [
            'text/plain',
            'text/html',
            'text/x-vhdl',
        ]
        if type in texttypes and not isAttachment:
            if messagetype == 'multipart/alternative' \
              and type != preferredtypes[props.create_format]:
                pass
            else:
                data = part.data
                
                # TMP code
                if data is None:
                    print 'data is None !!'
                
                if encoding == 'quoted-printable':
                    data = data.decode('quoted-printable').decode(charset)
                
                if isCreation:
                    useHtml = wysiwygSupport.checkCreateFormat('html')
                    if not isAttachment:
                        if useHtml:
                            if type in ['text/plain', 'text/x-vhdl']:
                                data = transform.convert('text_to_html',
                                                         data).getData()
                            if type == 'text/html':
                                data = transform.convert('safe_html',
                                                         data).getData()
                        else:
                            if type == 'text/html':
                                data = transform.convert('html_to_text',
                                                         data).getData()
                        
                if type in ['text/plain', 'text/x-vhdl'] and not isCreation:
                    data = transform.convert('text_to_html', data).getData()
                
                if type == 'text/html' and not isCreation:
                    data = transform.convert('safe_html', data).getData()
        
        url = None
        ignoretypes = [
            'message/delivery-status',
            'text/rfc822-headers',
            'unknown',
        ]    
        if isAttachment and not type in ignoretypes:
            # TODO: do something with createMode
            url = self._handleAttachment(part)
            if type.find('image') > -1:
                data = '<img src="' + url + '/image_preview" alt="" />'
            elif type != 'text/plain' and type != 'text/html':
                data = ''
        
        if type == 'text/rfc822-headers':
            if not isCreation:
                data = transform.convert('text_to_html', part.data).getData()
        
        if type == 'unknown':
            if not isCreation:
                data = transform.convert('text_to_html',
                                         part.part.as_string()).getData()
        
        messagepart = {
            'title': '',
            'type': type,
            'charset': charset,
            'isAttachment': isAttachment,
            'filename': filename,
            'name': name,
            'encoding': encoding,
            'data': data,
            'url': url,
            'subparts': list(),
            'headers': dict(),
            'media': None,
        }
        
        if messagetype == 'message/rfc822':
            headers = messagepart['headers']
            headers['to'] = decodeHeader(part.getHeader('To'))
            headers['from'] = decodeHeader(part.getHeader('From'))
            headers['cc'] = decodeHeader(part.getHeader('Cc'))
            headers['date'] = part.getHeader('Date')
            subject = decodeHeader(part.getHeader('Subject'))
            headers['subject'] = subject
            messagepart['title'] = 'E-Mail Nachricht - %s' % subject
        
        if type in self.audiotypes or type in self.videotypes:
            if type in self.audiotypes:
                messagepart['media'] = 'audioembed'
            if type in self.videotypes:
                messagepart['media'] = 'videoembed'
        
        messagetype = part.part.get_content_type()
        for subpart in part.subparts:
            respart = self._resolveMessageParts(subpart, isCreation,
                                                wysiwygSupport, props,
                                                preferredtypes, messagetype,
                                                transform)
            messagepart['subparts'].append(respart)
        
        return messagepart
    
    def _handleAttachment(self, part):
        """Take a MessagePart Object witch is an Attachment as argument.
        Look up for the attachment container and store this attachment to it.
        
        Looks up in mailbox Object if there is a Attachment stored with the
        same ID. If not, store the attachment to the mailbox Object.
        finally store the url of the attachment in part headers.
        
        TODO: This behaviour act a little like caching. so there must be done
        a little research on imap how to read just the message part headers if
        this is possible and do the download just when the Attachment is not
        downloaded yet. the alternative to this is to store the whole message 
        at the first time downloaded and and look for it before download
        the entire message. but this has to be done at another place.
        at this place a object expiration must be defined to prevent the 
        mailbox growing too big.
        """
        # generate the attachment id
        folder = self.request.get('folder')
        imapId = self.request.get('imapId')
        name = part.filename
        if name == None:
            name = part.name
        id = folder + imapId + name
        
        #lookup for the attachment container
        container = self.context
        containerpath = self.context.getAttachmentContainer().strip()
        containertype = self.context.getAttachmentContainerType().strip()
        if containerpath != '':
            container = self.context.portal_url.getPortalObject()
            containerpath = containerpath.split('/')
            for objectpath in containerpath:
                last = container
                container = container.get(objectpath, None)
                if container is None:
                    last.invokeFactory(containertype,
                                       objectpath,
                                       title='E-Mail Attachments')
                    container = last.get(objectpath)
        
        self._cleanupAttachmentContainer(container)
            
        # TODO: Complete this listing with more filetypes, f.e. audio files.
        type = part.contenttype.lower()
        if type.find('image') > -1:
            type = 'Image'
        else:
            type = 'File'
        
        ret = '%s/%s' % (container.absolute_url(), id)
        attachment = container.get(id, None)
        if attachment:
            return ret
        
        #create the file object
        file = File(id=id,
                    title=name,
                    file=part.data,
                    content_type=part.contenttype)
        #check for file type and write to attachment container
        if type == 'File':
            container.invokeFactory(type, id=id, title=name,
                                    format=part.contenttype, file=file)
        if type == 'Image':
            container.invokeFactory(type, id=id, title=name,
                                   format=part.contenttype, image=file)
        
        return ret
    
    def _cleanupAttachmentContainer(self, container):
        """Check max attachment of attachment container and cleanup if
        neccassary.
        """
        containerid = container.getId()
        maxsize = self.context.getAttachmentContainerMaxSize()
        catalog = self.context.portal_catalog
        query = {
            'sort_on': 'modified',
            'sort_order': 'normal',
            'path': {'query': '/'.join(container.getPhysicalPath()), 'depth': 1}
        }
        brains = catalog(**query)
        objectcount = len(brains)
        if objectcount >= maxsize:
            deletecount = objectcount - maxsize + 1
            todelete = []
            for brain in brains:
                todelete.append(brain.getId)
                deletecount -= 1
                if deletecount == 0:
                    break
            container.manage_delObjects(todelete)
        return
