# -*- coding: utf-8 -*-
#
# Copyright (c) 2006-2007 by:
#     Blue Dynamics Alliance
#         * Klein & Partner KEG, Austria
#         * Squarewave Computing Robert Niederreiter, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Robert Niederreiter <rnix@squarewave.at>"""
__docformat__ = 'plaintext'

"""Serversettings for several IMAP Servers
    
First of all, Thanks to the squirrelmail team for this informations
about the different behaviour of IMAP servers with foldermanagement by
providing all those nice presets. They were originaly taken 1:1 from
http://www.squirrelmail.org/docs/admin/admin-12.html and redefined here for
use in python.

NOTE: not all of the flags set by the presets are observed at the moment,
cause i have not the time to implement all those features atm, and, of course,
not even an idea what should some of them values effect. contact
the author when you want to implement support for anything not done yet
to get more information about this software. all non supported flags and
non-tested servers are listed below.

The following Servers are tested:
    * Courier IMAP server
    * Cyrus IMAP server

Following Servers are planned to be supported, so, not tested yet:
    * Binc IMAP server
    * Dovecot Secure IMAP server
    * Microsoft Exchange IMAP server
    * hMailServer
    * Mac OS X Mailserver
    * Mercury/32
    * University of Washington's IMAP server

This are the configuration flags, not all of them are observed yet:
    * default_folder_prefix
    * trash_folder
    * sent_folder
    * draft_folder
    * show_prefix_option
    * default_sub_of_inbox -> maybe get rid of this in next version, we'll see
    * show_contain_subfolders_option
    * optional_delimiter
    * delete_folder
    * force_username_lowercase
    * folder_creation_encoding
"""

IMAPSERVER_TYPES = {
    'default': 'Default IMAP Server Settings',
    'courier': 'Courier IMAP server',
    'cyrus': 'Cyrus IMAP server',
    'bincimap1.2.13': 'Binc 1.2.13 IMAP server',
    'bincimap1.3.4': 'Binc 1.3.4 IMAP server',
    'dovecot': 'Dovecot Secure IMAP server',
    'exchange': 'Microsoft Exchange IMAP server',
    'hmailserver': 'hMailServer',
    'macosx': 'Mac OS X Mailserver',
    'mercury32': 'Mercury/32',
    'uw': 'University of Washington\'s IMAP server',
}

IMAPSERVER_PRESETS = {
    'default': {
        'folder_prefix': '',
        'trash_folder': 'INBOX.Trash',
        'sent_folder': 'INBOX.Sent',
        'draft_folder': 'INBOX.Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': True,
        'show_contain_subfolders_option': False,
        'delimiter': 'detect',
        'delete_folder': False,
        'force_username_lowercase': False,
        'folder_creation_encoding': 'utf-8',
    },
    'courier': {
        'folder_prefix': 'INBOX.',
        'trash_folder': 'Trash',
        'sent_folder': 'Sent',
        'draft_folder': 'Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': False,
        'show_contain_subfolders_option': False,
        'delimiter': '.',
        'delete_folder': True,
        'force_username_lowercase': False,
        'folder_creation_encoding': 'utf-8',
    },
    'cyrus': {
        'folder_prefix': '',
        'trash_folder': 'INBOX.Trash',
        'sent_folder': 'INBOX.Sent',
        'draft_folder': 'INBOX.Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': True,
        'show_contain_subfolders_option': False,
        'delimiter': '.',
        'delete_folder': True,
        'force_username_lowercase': False,
        'folder_creation_encoding': 'ascii',
    },
    'bincimap1.2.13': {
        'folder_prefix': 'INBOX/',
        'trash_folder': 'Trash',
        'sent_folder': 'Sent',
        'draft_folder': 'Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': False,
        'show_contain_subfolders_option': False,
        'delimiter': '/', # can be set to 'detect'
        'delete_folder': True,
        'force_username_lowercase': False, # depends on checkpassword
        'folder_creation_encoding': 'utf-8',
    },
    'bincimap1.3.4': {
        'folder_prefix': '',
        'trash_folder': 'Trash',
        'sent_folder': 'Sent',
        'draft_folder': 'Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': False,
        'show_contain_subfolders_option': False,
        'delimiter': '/', #can be set to 'detect'
        'delete_folder': True,
        'force_username_lowercase': False, # depends on checkpassword
        'folder_creation_encoding': 'utf-8',
    },
    'dovecot': {
        'folder_prefix': '',
        'trash_folder': 'Trash',
        'sent_folder': 'Sent',
        'draft_folder': 'Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': False,
        'show_contain_subfolders_option': False,
        'delimiter': 'detect',
        'delete_folder': False,
        'force_username_lowercase': True,
        'folder_creation_encoding': 'utf-8',
    },
    'exchange': {
        'folder_prefix': '',
        'trash_folder': 'INBOX/Deleted Items',
        'sent_folder': 'INBOX/Sent Items',
        'draft_folder': 'Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': True,
        'show_contain_subfolders_option': False,
        'delimiter': 'detect',
        'delete_folder': True,
        'force_username_lowercase': True,
        'folder_creation_encoding': 'utf-8',
    },
    'hmailserver': {
        'folder_prefix': '',
        'trash_folder': 'INBOX.Trash',
        'sent_folder': 'INBOX.Sent',
        'draft_folder': 'INBOX.Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': True,
        'show_contain_subfolders_option': False,
        'delimiter': 'detect',
        'delete_folder': False,
        'force_username_lowercase': False,
        'folder_creation_encoding': 'utf-8',
    },
    'macosx': {
        'folder_prefix': 'INBOX/',
        'trash_folder': 'Trash',
        'sent_folder': 'Sent',
        'draft_folder': 'Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': True,
        'show_contain_subfolders_option': False,
        'delimiter': 'detect',
        'delete_folder': True,
        'force_username_lowercase': False,
        'folder_creation_encoding': 'utf-8',
    },
    'mercury32': {
        'folder_prefix': '',
        'trash_folder': 'Trash',
        'sent_folder': 'Sent',
        'draft_folder': 'Drafts',
        'show_prefix_option': False,
        'default_sub_of_inbox': True,
        'show_contain_subfolders_option': True,
        'delimiter': 'detect',
        'delete_folder': True,
        'force_username_lowercase': True,
        'folder_creation_encoding': 'utf-8',
    },
    'uw': {
        'folder_prefix': 'mail/',
        'trash_folder': 'Trash',
        'sent_folder': 'Sent',
        'draft_folder': 'Drafts',
        'show_prefix_option': True,
        'default_sub_of_inbox': False,
        'show_contain_subfolders_option': True,
        'delimiter': '/',
        'delete_folder': True,
        'force_username_lowercase': True,
        'folder_creation_encoding': 'utf-8',
    },
}